@@ -18,12 +18,13 @@ import (
 	"encoding/json"
 	"fmt"
 	"reflect"
+	"runtime"
 	"sort"
 	"sync"
 	"testing"
 	"time"
 
-	"github.com/coreos/fleet/Godeps/_workspace/src/github.com/jonboulle/clockwork"
+	"github.com/jonboulle/clockwork"
 
 	"github.com/coreos/fleet/machine"
 	"github.com/coreos/fleet/registry"
@@ -319,7 +320,7 @@ func TestUnitStatePublisherRunTiming(t *testing.T) {
 	}
 
 	bc := make(chan *unit.UnitStateHeartbeat)
-	sc := make(chan bool)
+	sc := make(chan struct{})
 	go func() {
 		usp.Run(bc, sc)
 	}()
@@ -465,7 +466,7 @@ func TestUnitStatePublisherRunQueuing(t *testing.T) {
 		clock:           clockwork.NewFakeClock(),
 	}
 	bc := make(chan *unit.UnitStateHeartbeat)
-	sc := make(chan bool)
+	sc := make(chan struct{})
 	go func() {
 		usp.Run(bc, sc)
 	}()
@@ -568,6 +569,9 @@ func TestUnitStatePublisherRunQueuing(t *testing.T) {
 }
 
 func TestUnitStatePublisherRunWithDelays(t *testing.T) {
+	if runtime.GOMAXPROCS(-1) != 1 {
+		t.Skipf("Broken for GOMAXPROCS != 1, currently %d", runtime.GOMAXPROCS(-1))
+	}
 	states := make([]string, 0)
 	mu := sync.Mutex{} // protects states from concurrent modification
 	fclock := clockwork.NewFakeClock()
@@ -595,7 +599,7 @@ func TestUnitStatePublisherRunWithDelays(t *testing.T) {
 	}
 
 	bc := make(chan *unit.UnitStateHeartbeat)
-	sc := make(chan bool)
+	sc := make(chan struct{})
 
 	wgs.Add(numPublishers)
 	wgf.Add(numPublishers)
@@ -647,7 +651,15 @@ func TestUnitStatePublisherRunWithDelays(t *testing.T) {
 
 	usp.toPublishMutex.RLock()
 	if !reflect.DeepEqual(usp.toPublishStates, wtps) {
-		t.Errorf("bad toPublishStates: got %#v, want %#v", usp.toPublishStates, wtps)
+		t.Errorf("bad toPublishStates")
+		t.Logf("got:\n")
+		for k, v := range usp.toPublishStates {
+			t.Logf("  %v = %#v", k, v)
+		}
+		t.Logf("want:\n")
+		for k, v := range wtps {
+			t.Logf("  %v = %#v", k, v)
+		}
 	}
 	usp.toPublishMutex.RUnlock()
 
