@@ -43,7 +43,7 @@ func newUnit(t *testing.T, str string) unit.UnitFile {
 func TestUnitsSubResourceNotFound(t *testing.T) {
 	fr := registry.NewFakeRegistry()
 	fAPI := &client.RegistryClient{Registry: fr}
-	ur := &unitsResource{fAPI, "/units"}
+	ur := &unitsResource{fAPI, "/units", testTokenLimit}
 	rr := httptest.NewRecorder()
 
 	req, err := http.NewRequest("GET", "/units/foo/bar", nil)
@@ -66,7 +66,7 @@ func TestUnitsList(t *testing.T) {
 		{Name: "YYY.service"},
 	})
 	fAPI := &client.RegistryClient{Registry: fr}
-	resource := &unitsResource{fAPI, "/units"}
+	resource := &unitsResource{fAPI, "/units", testTokenLimit}
 	rw := httptest.NewRecorder()
 	req, err := http.NewRequest("GET", "http://example.com/units", nil)
 	if err != nil {
@@ -103,7 +103,7 @@ func TestUnitsList(t *testing.T) {
 func TestUnitsListBadNextPageToken(t *testing.T) {
 	fr := registry.NewFakeRegistry()
 	fAPI := &client.RegistryClient{Registry: fr}
-	resource := &unitsResource{fAPI, "/units"}
+	resource := &unitsResource{fAPI, "/units", testTokenLimit}
 	rw := httptest.NewRecorder()
 	req, err := http.NewRequest("GET", "http://example.com/units?nextPageToken=EwBMLg==", nil)
 	if err != nil {
@@ -184,7 +184,7 @@ func TestUnitGet(t *testing.T) {
 		{Name: "YYY.service"},
 	})
 	fAPI := &client.RegistryClient{Registry: fr}
-	resource := &unitsResource{fAPI, "/units"}
+	resource := &unitsResource{fAPI, "/units", testTokenLimit}
 
 	for i, tt := range tests {
 		rw := httptest.NewRecorder()
@@ -247,7 +247,7 @@ func TestUnitsDestroy(t *testing.T) {
 		}
 
 		fAPI := &client.RegistryClient{Registry: fr}
-		resource := &unitsResource{fAPI, "/units"}
+		resource := &unitsResource{fAPI, "/units", testTokenLimit}
 		rw := httptest.NewRecorder()
 		resource.destroy(rw, req, tt.arg)
 
@@ -415,7 +415,7 @@ func TestUnitsSetDesiredState(t *testing.T) {
 		req.Header.Set("Content-Type", "application/json")
 
 		fAPI := &client.RegistryClient{Registry: fr}
-		resource := &unitsResource{fAPI, "/units"}
+		resource := &unitsResource{fAPI, "/units", testTokenLimit}
 		rw := httptest.NewRecorder()
 		resource.set(rw, req, tt.item)
 
@@ -454,6 +454,14 @@ func makeConflictUO(name string) *schema.UnitOption {
 	}
 }
 
+func makeReplaceUO(name string) *schema.UnitOption {
+	return &schema.UnitOption{
+		Section: "X-Fleet",
+		Name:    "Replaces",
+		Value:   name,
+	}
+}
+
 func makePeerUO(name string) *schema.UnitOption {
 	return &schema.UnitOption{
 		Section: "X-Fleet",
@@ -543,6 +551,39 @@ func TestValidateOptions(t *testing.T) {
 			},
 			false,
 		},
+		// Replaces
+		// non-overlapping replace is fine
+		{
+			[]*schema.UnitOption{
+				makeReplaceUO("foo.service"),
+				makeReplaceUO("bar.service"),
+			},
+			true,
+		},
+		{
+			[]*schema.UnitOption{
+				makeReplaceUO("foo.service"),
+				makePeerUO("bar.service"),
+			},
+			true,
+		},
+		// replace + conflict is not good
+		{
+			[]*schema.UnitOption{
+				makeReplaceUO("foo.service"),
+				makeConflictUO("bar.service"),
+			},
+			false,
+		},
+		// circular replaces are not good
+		{
+			[]*schema.UnitOption{
+				makeReplaceUO("foo.service"),
+				makeReplaceUO("bar.service"),
+				makePeerUO("bar.service"),
+			},
+			false,
+		},
 		// MachineID is fine by itself
 		{
 			[]*schema.UnitOption{
@@ -576,7 +617,25 @@ func TestValidateOptions(t *testing.T) {
 		},
 		{
 			[]*schema.UnitOption{
-				makeIDUO("zyxwvutsr"), makeConflictUO("foo.service"), makeConflictUO("bar.service"),
+				makeIDUO("zyxwvutsr"),
+				makeConflictUO("foo.service"),
+				makeConflictUO("bar.service"),
+			},
+			false,
+		},
+		// MachineID with Replaces no good
+		{
+			[]*schema.UnitOption{
+				makeIDUO("abcdefghi"),
+				makeReplaceUO("bar.service"),
+			},
+			false,
+		},
+		{
+			[]*schema.UnitOption{
+				makeIDUO("zyxwvutsr"),
+				makeReplaceUO("foo.service"),
+				makeReplaceUO("bar.service"),
 			},
 			false,
 		},
@@ -649,6 +708,18 @@ func TestValidateOptions(t *testing.T) {
 			},
 			false,
 		},
+		// Global with Replaces no good
+		{
+			[]*schema.UnitOption{
+				&schema.UnitOption{
+					Section: "X-Fleet",
+					Name:    "Global",
+					Value:   "true",
+				},
+				makeReplaceUO("foo.service"),
+			},
+			false,
+		},
 	}
 	for i, tt := range testCases {
 		err := ValidateOptions(tt.opts)
@@ -718,7 +789,7 @@ func TestValidateName(t *testing.T) {
 func TestUnitsSetDesiredStateBadContentType(t *testing.T) {
 	fr := registry.NewFakeRegistry()
 	fAPI := &client.RegistryClient{Registry: fr}
-	resource := &unitsResource{fAPI, "/units"}
+	resource := &unitsResource{fAPI, "/units", testTokenLimit}
 	rr := httptest.NewRecorder()
 
 	body := ioutil.NopCloser(bytes.NewBuffer([]byte(`{"foo":"bar"}`)))
