@@ -30,6 +30,9 @@ type Config struct {
 	Verbosity               int
 	RawMetadata             string
 	AgentTTL                string
+	TokenLimit              int
+	DisableEngine           bool
+	DisableWatches          bool
 	VerifyUnits             bool
 	AuthorizedKeysFile      string
 }
