@@ -14,31 +14,42 @@
 
 package main
 
-var cmdSubmitUnit = &Command{
-	Name:    "submit",
-	Summary: "Upload one or more units to the cluster without starting them",
-	Usage:   "UNIT...",
-	Description: `Upload one or more units to the cluster without starting them. Useful
+import (
+	"github.com/spf13/cobra"
+)
+
+var cmdSubmit = &cobra.Command{
+	Use:   "submit UNIT...",
+	Short: "Upload one or more units to the cluster without starting them",
+	Long: `Upload one or more units to the cluster without starting them. Useful
 for validating units before they are started.
 
 This operation is idempotent; if a named unit already exists in the cluster, it will not be resubmitted.
 
 Submit a single unit:
-	fleetctl submit foo.service
+fleetctl submit foo.service
 
 Submit a directory of units with glob matching:
-	fleetctl submit myservice/*`,
-	Run: runSubmitUnits,
+fleetctl submit myservice/*`,
+	Run: runWrapper(runSubmitUnit),
 }
 
 func init() {
-	cmdSubmitUnit.Flags.BoolVar(&sharedFlags.Sign, "sign", false, "DEPRECATED - this option cannot be used")
+	cmdFleet.AddCommand(cmdSubmit)
+
+	cmdSubmit.Flags().BoolVar(&sharedFlags.Sign, "sign", false, "DEPRECATED - this option cannot be used")
+	cmdSubmit.Flags().BoolVar(&sharedFlags.Replace, "replace", false, "Replace the old submitted units in the cluster with new versions.")
 }
 
-func runSubmitUnits(args []string) (exit int) {
-	if err := lazyCreateUnits(args); err != nil {
-		stderr("Error creating units: %v", err)
-		exit = 1
+func runSubmitUnit(cCmd *cobra.Command, args []string) (exit int) {
+	if len(args) == 0 {
+		stderr("No units given")
+		return 0
 	}
-	return
+
+	if err := lazyCreateUnits(cCmd, args); err != nil {
+		stderr("Error creating units: %v", err)
+		return 1
+	}
+	return 0
 }
