@@ -24,12 +24,14 @@ import (
 	"os"
 	"os/exec"
 	"path"
+	"regexp"
+	"runtime"
 	"strconv"
 	"strings"
+	"testing"
 	"time"
 
-	"github.com/coreos/fleet/Godeps/_workspace/src/code.google.com/p/go-uuid/uuid"
-	"github.com/coreos/fleet/Godeps/_workspace/src/github.com/coreos/go-systemd/dbus"
+	"github.com/coreos/go-systemd/dbus"
 
 	"github.com/coreos/fleet/functional/util"
 )
@@ -39,9 +41,11 @@ const (
 )
 
 var fleetdBinPath string
+var fleetctlBinPath string
 
 func init() {
 	fleetdBinPath = os.Getenv("FLEETD_BIN")
+	fleetctlBinPath = os.Getenv("FLEETCTL_BIN")
 	if fleetdBinPath == "" {
 		fmt.Println("FLEETD_BIN environment variable must be set")
 		os.Exit(1)
@@ -49,6 +53,13 @@ func init() {
 		fmt.Printf("%v\n", err)
 		os.Exit(1)
 	}
+	if fleetctlBinPath == "" {
+		fmt.Println("FLEETCTL_BIN environment variable must be set")
+		os.Exit(1)
+	} else if _, err := os.Stat(fleetctlBinPath); err != nil {
+		fmt.Printf("%v\n", err)
+		os.Exit(1)
+	}
 	// sanity check etcd availability
 	cmd := exec.Command("etcdctl", "ls")
 	out, err := cmd.CombinedOutput()
@@ -94,34 +105,82 @@ func (nc *nspawnCluster) keyspace() string {
 	return fmt.Sprintf("/fleet_functional/%s", nc.name)
 }
 
+// This function adds --endpoint flag if --tunnel flag is not used
+// Usefull for "fleetctl fd-forward" tests
+func handleEndpointFlag(m Member, args *[]string) {
+	result := true
+	for _, arg := range *args {
+		if strings.Contains(arg, "-- ") || strings.Contains(arg, "--tunnel") {
+			result = false
+			break
+		}
+	}
+	if result {
+		*args = append([]string{"--endpoint=" + m.Endpoint()}, *args...)
+	}
+}
+
 func (nc *nspawnCluster) Fleetctl(m Member, args ...string) (string, string, error) {
-	args = append([]string{"--endpoint=" + m.Endpoint()}, args...)
+	handleEndpointFlag(m, &args)
 	return util.RunFleetctl(args...)
 }
 
 func (nc *nspawnCluster) FleetctlWithInput(m Member, input string, args ...string) (string, string, error) {
-	args = append([]string{"--endpoint=" + m.Endpoint()}, args...)
+	handleEndpointFlag(m, &args)
 	return util.RunFleetctlWithInput(input, args...)
 }
 
+// WaitForNUnits runs fleetctl list-units to verify the actual number of units
+// matched with the given expected number. It periodically runs list-units
+// waiting until list-units actually shows the expected units.
+func (nc *nspawnCluster) WaitForNUnits(m Member, expectedUnits int) (map[string][]util.UnitState, error) {
+	var nUnits int
+	retStates := make(map[string][]util.UnitState)
+	checkListUnits := func() bool {
+		outListUnits, _, err := nc.Fleetctl(m, "list-units", "--no-legend", "--full", "--fields", "unit,active,machine")
+		if err != nil {
+			return false
+		}
+		// NOTE: There's no need to check if outListUnits is expected to be empty,
+		// because ParseUnitStates() implicitly filters out such cases.
+		// However, in case of ParseUnitStates() going away, we should not
+		// forget about such special cases.
+		units := strings.Split(strings.TrimSpace(outListUnits), "\n")
+		allStates := util.ParseUnitStates(units)
+		nUnits = len(allStates)
+		if nUnits != expectedUnits {
+			return false
+		}
+
+		for _, state := range allStates {
+			name := state.Name
+			if _, ok := retStates[name]; !ok {
+				retStates[name] = []util.UnitState{}
+			}
+			retStates[name] = append(retStates[name], state)
+		}
+		return true
+	}
+
+	timeout, err := util.WaitForState(checkListUnits)
+	if err != nil {
+		return nil, fmt.Errorf("failed to find %d units within %v (last found: %d)",
+			expectedUnits, timeout, nUnits)
+	}
+
+	return retStates, nil
+}
+
 func (nc *nspawnCluster) WaitForNActiveUnits(m Member, count int) (map[string][]util.UnitState, error) {
 	var nactive int
 	states := make(map[string][]util.UnitState)
 
-	timeout := 15 * time.Second
-	alarm := time.After(timeout)
-
-	ticker := time.Tick(250 * time.Millisecond)
-loop:
-	for {
-		select {
-		case <-alarm:
-			return nil, fmt.Errorf("failed to find %d active units within %v (last found: %d)", count, timeout, nactive)
-		case <-ticker:
+	timeout, err := util.WaitForState(
+		func() bool {
 			stdout, _, err := nc.Fleetctl(m, "list-units", "--no-legend", "--full", "--fields", "unit,active,machine")
 			stdout = strings.TrimSpace(stdout)
 			if err != nil {
-				continue
+				return false
 			}
 
 			lines := strings.Split(stdout, "\n")
@@ -129,7 +188,7 @@ loop:
 			active := util.FilterActiveUnits(allStates)
 			nactive = len(active)
 			if nactive != count {
-				continue
+				return false
 			}
 
 			for _, state := range active {
@@ -139,13 +198,59 @@ loop:
 				}
 				states[name] = append(states[name], state)
 			}
-			break loop
-		}
+			return true
+		},
+	)
+	if err != nil {
+		return nil, fmt.Errorf("failed to find %d active units within %v (last found: %d)", count, timeout, nactive)
 	}
 
 	return states, nil
 }
 
+// WaitForNUnitFiles runs fleetctl list-unit-files to verify the actual number of units
+// matched with the given expected number. It periodically runs list-unit-files
+// waiting until list-unit-files actually shows the expected units.
+func (nc *nspawnCluster) WaitForNUnitFiles(m Member, expectedUnits int) (map[string][]util.UnitFileState, error) {
+	var nUnits int
+	retStates := make(map[string][]util.UnitFileState)
+
+	checkListUnitFiles := func() bool {
+		outListUnitFiles, _, err := nc.Fleetctl(m, "list-unit-files", "--no-legend", "--full", "--fields", "unit,dstate,state")
+		if err != nil {
+			return false
+		}
+		// NOTE: There's no need to check if outListUnits is expected to be empty,
+		// because ParseUnitFileStates() implicitly filters out such cases.
+		// However, in case of ParseUnitFileStates() going away, we should not
+		// forget about such special cases.
+		units := strings.Split(strings.TrimSpace(outListUnitFiles), "\n")
+		allStates := util.ParseUnitFileStates(units)
+		nUnits = len(allStates)
+		if nUnits != expectedUnits {
+			// retry until number of units matched
+			return false
+		}
+
+		for _, state := range allStates {
+			name := state.Name
+			if _, ok := retStates[name]; !ok {
+				retStates[name] = []util.UnitFileState{}
+			}
+			retStates[name] = append(retStates[name], state)
+		}
+		return true
+	}
+
+	timeout, err := util.WaitForState(checkListUnitFiles)
+	if err != nil {
+		return nil, fmt.Errorf("failed to find %d units within %v (last found: %d)",
+			expectedUnits, timeout, nUnits)
+	}
+
+	return retStates, nil
+}
+
 func (nc *nspawnCluster) WaitForNMachines(m Member, count int) ([]string, error) {
 	var machines []string
 	timeout := 10 * time.Second
@@ -211,10 +316,10 @@ func (nc *nspawnCluster) prepCluster() (err error) {
 		return
 	}
 
-	if !strings.Contains(stdout, "172.17.0.1/16") {
-		_, _, err = run("ip addr add 172.17.0.1/16 dev fleet0")
+	if !strings.Contains(stdout, "172.18.0.1/16") {
+		_, _, err = run("ip addr add 172.18.0.1/16 dev fleet0")
 		if err != nil {
-			log.Printf("Failed adding 172.17.0.1/16 to fleet0: %v", err)
+			log.Printf("Failed adding 172.18.0.1/16 to fleet0: %v", err)
 			return
 		}
 	}
@@ -228,18 +333,18 @@ func (nc *nspawnCluster) prepCluster() (err error) {
 	return nil
 }
 
-func (nc *nspawnCluster) insertFleetd(dir string) error {
-	cmd := fmt.Sprintf("mkdir -p %s/opt/fleet", dir)
+func (nc *nspawnCluster) insertBin(src string, dst string) error {
+	cmd := fmt.Sprintf("mkdir -p %s/opt/fleet", dst)
 	if _, _, err := run(cmd); err != nil {
 		return err
 	}
 
-	fleetdBinDst := path.Join(dir, "opt", "fleet", "fleetd")
-	return copyFile(fleetdBinPath, fleetdBinDst, 0755)
+	binDst := path.Join(dst, "opt", "fleet", path.Base(src))
+	return copyFile(src, binDst, 0755)
 }
 
 func (nc *nspawnCluster) buildConfigDrive(dir, ip string) error {
-	latest := path.Join(dir, "media/configdrive/openstack/latest")
+	latest := path.Join(dir, "var/lib/coreos-install")
 	userPath := path.Join(latest, "user_data")
 	if err := os.MkdirAll(latest, 0755); err != nil {
 		return err
@@ -251,7 +356,7 @@ func (nc *nspawnCluster) buildConfigDrive(dir, ip string) error {
 	}
 	defer userFile.Close()
 
-	etcd := "http://172.17.0.1:4001"
+	etcd := "http://172.18.0.1:4001"
 	return util.BuildCloudConfig(userFile, ip, etcd, nc.keyspace())
 }
 
@@ -281,16 +386,11 @@ func (nc *nspawnCluster) CreateMember() (m Member, err error) {
 	return nc.createMember(id)
 }
 
-func newMachineID() string {
-	// drop the standard separators to match systemd
-	return strings.Replace(uuid.New(), "-", "", -1)
-}
-
 func (nc *nspawnCluster) createMember(id string) (m Member, err error) {
 	nm := nspawnMember{
-		uuid: newMachineID(),
+		uuid: util.NewMachineID(),
 		id:   id,
-		ip:   fmt.Sprintf("172.17.1.%s", id),
+		ip:   fmt.Sprintf("172.18.1.%s", id),
 	}
 	nc.members[nm.ID()] = nm
 
@@ -300,16 +400,23 @@ func (nc *nspawnCluster) createMember(id string) (m Member, err error) {
 		// set up directory for fleet service
 		fmt.Sprintf("mkdir -p %s/etc/systemd/system", fsdir),
 
-		// minimum requirements for running systemd/coreos in a container
+		// minimum requirements for running systemd/coreos in a container.
+		// NOTE: copying /etc/pam.d is necessary only for such setups with
+		// sys-auth/pambase installed, for example developer image of CoreOS
+		// 1053.2.0. It should be fine also for systems where /etc/pam.d is
+		// empty, because then it should automatically fall back to
+		// /usr/lib64/pam.d, which belongs to sys-libs/pam.
 		fmt.Sprintf("mkdir -p %s/usr", fsdir),
 		fmt.Sprintf("cp /etc/os-release %s/etc", fsdir),
+		fmt.Sprintf("cp -a /etc/pam.d %s/etc", fsdir),
 		fmt.Sprintf("ln -s /proc/self/mounts %s/etc/mtab", fsdir),
 		fmt.Sprintf("ln -s usr/lib64 %s/lib64", fsdir),
 		fmt.Sprintf("ln -s lib64 %s/lib", fsdir),
 		fmt.Sprintf("ln -s usr/bin %s/bin", fsdir),
 		fmt.Sprintf("ln -s usr/sbin %s/sbin", fsdir),
 		fmt.Sprintf("mkdir -p %s/home/core/.ssh", fsdir),
-		fmt.Sprintf("chown -R core:core %s/home/core", fsdir),
+		fmt.Sprintf("install -d -o root -g systemd-journal -m 2755 %s/var/log/journal", fsdir),
+		fmt.Sprintf("chown -R 500:500 %s/home/core", fsdir),
 
 		// We don't need this, and it's slow, so mask it
 		fmt.Sprintf("ln -s /dev/null %s/etc/systemd/system/systemd-udev-hwdb-update.service", fsdir),
@@ -327,36 +434,72 @@ func (nc *nspawnCluster) createMember(id string) (m Member, err error) {
 		}
 	}
 
-	sshd_config := `# Use most defaults for sshd configuration.
-UsePrivilegeSeparation sandbox
-Subsystem sftp internal-sftp
-UseDNS no
-`
+	filesContents := []struct {
+		path     string
+		contents string
+		mode     os.FileMode
+	}{
+		{
+			"/etc/ssh/sshd_config",
+			`# Use most defaults for sshd configuration.
+			UsePrivilegeSeparation sandbox
+			Subsystem sftp internal-sftp
+			UseDNS no
+			`,
+			0644,
+		},
+		// For expediency, generate the minimal viable SSH keys for the host, instead of the default set
+		{
+			"/etc/systemd/system/sshd-keygen.service",
+			`[Unit]
+			Description=Generate sshd host keys
+			Before=sshd.service
 
-	if err = ioutil.WriteFile(path.Join(fsdir, "/etc/ssh/sshd_config"), []byte(sshd_config), 0644); err != nil {
-		log.Printf("Failed writing sshd_config: %v", err)
-		return
+			[Service]
+			Type=oneshot
+			RemainAfterExit=yes
+			ExecStart=/usr/bin/ssh-keygen -t rsa -f /etc/ssh/ssh_host_rsa_key -N "" -b 1024`,
+			0644,
+		},
+		{
+			"/etc/passwd",
+			"core:x:500:500:CoreOS Admin:/home/core:/bin/bash",
+			0644,
+		},
+		{
+			"/etc/group",
+			"core:x:500:",
+			0644,
+		},
+		{
+			"/etc/machine-id",
+			nm.ID(),
+			0644,
+		},
+		{
+			"/home/core/.bash_profile",
+			"export PATH=/opt/fleet:$PATH",
+			0644,
+		},
 	}
 
-	// For expediency, generate the minimal viable SSH keys for the host, instead of the default set
-	sshd_keygen := `[Unit]
-	Description=Generate sshd host keys
-	Before=sshd.service
-
-	[Service]
-	Type=oneshot
-	RemainAfterExit=yes
-	ExecStart=/usr/bin/ssh-keygen -t rsa -f /etc/ssh/ssh_host_rsa_key -N "" -b 768`
-	if err = ioutil.WriteFile(path.Join(fsdir, "/etc/systemd/system/sshd-keygen.service"), []byte(sshd_keygen), 0644); err != nil {
-		log.Printf("Failed writing sshd-keygen.service: %v", err)
-		return
+	for _, file := range filesContents {
+		if err = ioutil.WriteFile(path.Join(fsdir, file.path), []byte(file.contents), file.mode); err != nil {
+			log.Printf("Failed writing %s: %v", path.Join(fsdir, file.path), err)
+			return
+		}
 	}
 
-	if err = nc.insertFleetd(fsdir); err != nil {
+	if err = nc.insertBin(fleetdBinPath, fsdir); err != nil {
 		log.Printf("Failed preparing fleetd in filesystem: %v", err)
 		return
 	}
 
+	if err = nc.insertBin(fleetctlBinPath, fsdir); err != nil {
+		log.Printf("Failed preparing fleetctl in filesystem: %v", err)
+		return
+	}
+
 	if err = nc.buildConfigDrive(fsdir, nm.IP()); err != nil {
 		log.Printf("Failed building config drive: %v", err)
 		return
@@ -395,6 +538,7 @@ UseDNS no
 			return
 		default:
 		}
+		log.Printf("Dialing machine: %s", addr)
 		c, err := net.DialTimeout("tcp", addr, 100*time.Millisecond)
 		if err == nil {
 			c.Close()
@@ -406,9 +550,36 @@ UseDNS no
 	return Member(&nm), nil
 }
 
-func (nc *nspawnCluster) Destroy() error {
+func (nc *nspawnCluster) Destroy(t *testing.T) error {
+	re := regexp.MustCompile(`/functional\.([a-zA-Z0-9]+)$`)
 	for _, m := range nc.Members() {
 		log.Printf("Destroying nspawn machine %s", m.ID())
+		if t.Failed() {
+			log.Printf("Failed tests, fetching logs from %s machine", m.ID())
+			wd, err := os.Getwd()
+			if err != nil {
+				log.Printf("Failed to get working directory, skipping journal logs fetch: %v", err)
+			} else {
+				var logPath string
+				containerDir := path.Join(os.TempDir(), nc.name, m.ID(), "fs")
+				stdout, _, _ := run(fmt.Sprintf("journalctl --directory=%s/var/log/journal --root=%s --no-pager", containerDir, containerDir))
+				pc := make([]uintptr, 10)
+				runtime.Callers(6, pc)
+				f := runtime.FuncForPC(pc[0])
+				match := re.FindStringSubmatch(f.Name())
+				if len(match) == 2 {
+					logPath = fmt.Sprintf("%s/%s_smoke%s.log", wd, match[1], m.ID())
+				} else {
+					logPath = fmt.Sprintf("%s/TestUnknown_smoke%s.log", wd, m.ID())
+				}
+				err = ioutil.WriteFile(logPath, []byte(stdout), 0644)
+				if err != nil {
+					log.Printf("Failed to write journal logs (%s): %v", logPath, err)
+				} else {
+					log.Printf("Wrote smoke%s logs to %s", m.ID(), logPath)
+				}
+			}
+		}
 		nc.DestroyMember(m)
 	}
 
@@ -431,15 +602,9 @@ func (nc *nspawnCluster) ReplaceMember(m Member) (Member, error) {
 	count := len(nc.members)
 	label := fmt.Sprintf("%s%s", nc.name, m.ID())
 
-	// The `machinectl poweroff` command does not cleanly shut down
-	// the nspawn container, so we must use systemctl
-	cmd := fmt.Sprintf("systemctl -M %s poweroff", label)
-	if _, stderr, _ := run(cmd); !strings.Contains(stderr, "Success") {
-		if strings.Contains(stderr, "Warning! D-Bus connection terminated.") {
-			log.Printf("poweroff failed: %s", stderr)
-		} else {
-			return nil, fmt.Errorf("poweroff failed: %s", stderr)
-		}
+	cmd := fmt.Sprintf("machinectl poweroff %s", label)
+	if _, _, err := run(cmd); err != nil {
+		return nil, fmt.Errorf("poweroff failed: %v", err)
 	}
 
 	var mN Member
